/* Compute log(2)/log(b), 2 <= b < 256.
 *
 * We compute logarithms using a rudimentary bit-at-a-time algorithm.
 * Numbers are kept as fixnums, scaled by 2^128.  We stick to dumbmp.c's
 * functions, e.g, we use mpz_root instead of mpz_sqrt.  Unfortunately,
 * using dumbmp.c makes things unusably slow; each base takes a few minutes
 * on a high-end machine.
 */

#include <stdlib.h>
#include <stdio.h>
#include "gmp.h"
// #include "dumbmp.c"

/* Compute b^e where the e and the result are fixnums. */
void
mp_exp (mpz_t r, int bi, mpz_t e)
{
  mpz_t x, b;
  int i;

  mpz_init_set_ui (b, bi);
  mpz_mul_2exp (b, b, 128);
  mpz_init_set_ui (x, 1);
  mpz_mul_2exp (x, x, 128);

  for (i = 63; i >= 0; i--)
    {
      mpz_mul_2exp (b, b, 128);
      mpz_root (b, b, 2);
      if (mpz_tstbit (e, i))
	{
	  mpz_mul (x, x, b);
	  mpz_tdiv_q_2exp (x, x, 128);
	}
    }

  mpz_set (r, x);
  mpz_clear (b);
  mpz_clear (x);
}

/* Compute log(2)/log(b) as a fixnum. */
void
mp_2log (mpz_t r, int b)
{
  mpz_t t, two, rold;
  int i;

  mpz_init (t);
  mpz_init (two);
  mpz_init (rold);
  mpz_set_ui (r, 0);
  mpz_set_ui (two, 2);
  mpz_mul_2exp (two, two, 128);

  for (i = 63; i >= 0; i--)
    {
      mpz_set (rold, r);		/* preserve last approximation */
      mpz_setbit (r, i);		/* set next less significant bit */
      mp_exp (t, b, r);			/* compute exp(b,r) */
      if (mpz_cmp (t, two) >= 0)	/* too large? */
	mpz_set (r, rold);		/* restore old value */
    }

  mpz_clear (t);
  mpz_clear (two);
  mpz_clear (rold);
}

int
main (int argc, char **argv)
{
  mpz_t r;
  int bi;

  mpz_init (r);

  for (bi = 2; bi < 256; bi++)
    {
      mp_2log (r, bi);
      mpz_out_str (stdout, 16, r);
      puts ("");
    }

  return 0;
}
